/*
 * Copyright (C) 2003 John David Ratliff
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.net.Socket;
import java.net.UnknownHostException;

/**
 * Connects to a simple file server and requests their file using
 * a signal message.
 */

public class Client {
    /**
     * The signal message.
     */
    public static final String SIGNAL = "send file\n";

    /**
     * Connect to the Server and request their file.
     *
     * @param args The server hostname and the port they are listening on.
     */
    public static void main(String[] args) {
        Socket socket;
        OutputStream out;
        InputStream in;
        int read;
        long count = 0, startTime, endTime;
        byte[] b = new byte[8192];
        double elapsed, Mbps;
        boolean single = false;
        
        try {
            // make the socket connection.
            socket = new Socket(args[0], Integer.parseInt(args[1]));
            
            if (args.length == 3) {
                // set single write mode
                System.out.println("Using single character read mode.");
                single = true;
            }
            
            // grab the I/O streams.
            in = socket.getInputStream();
            out = socket.getOutputStream();
            
            // send the signal message.
            out.write(SIGNAL.getBytes());
            
            // wait for a response.
            while (in.available() < 1);
            
            // grab the starting time
            startTime = System.currentTimeMillis();
            
            // send the file.
            if (single) {
                while (in.read() != -1) {
                    count++;
                }
            } else {
                while ((read = in.read(b,0,8192)) != -1) {
                    count += read;
                }
            }
            
            // grab the stop time
            endTime = System.currentTimeMillis();
            
            // close the socket
            socket.close();
            
            // calculate the elapsed time
            elapsed = (endTime - startTime) / 1000.0;
            
            // calculate the transfer speed
            Mbps = ((count * 8.0) / 1000000.0) / (elapsed > 0 ? elapsed : 1);
            
            // output the results to the terminal.
            System.out.println("Received " + count + " bytes at " + Mbps + " Mbps.");
        } catch (UnknownHostException e) {
            System.out.println("Could not find host " + args[0] + ".");
        } catch (IOException e) {
            System.out.println("Unable to open connection socket.");
            e.printStackTrace();
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("syntax: java Client server_name port");
        } catch (NumberFormatException e) {
            System.out.println(args[1] + " is not a valid port number.");
        }
    }
}
